@@ -22,6 +22,7 @@ from django.utils.safestring import mark_safe
 from django.views.generic import View
 from django_tables2 import RequestConfig
 
+from nautobot.core.forms import SearchForm
 from nautobot.extras.models import CustomField, ExportTemplate
 from nautobot.extras.models.change_logging import ChangeLoggedModel
 from nautobot.utilities.error_handlers import handle_protectederror
@@ -36,12 +37,15 @@ from nautobot.utilities.forms import (
     TableConfigForm,
     restrict_form_fields,
 )
+from nautobot.utilities.forms.forms import DynamicFilterFormSet
 from nautobot.utilities.paginator import EnhancedPaginator, get_paginate_count
 from nautobot.utilities.permissions import get_permission_for_model
-from nautobot.utilities.templatetags.helpers import validated_viewname
+from nautobot.utilities.templatetags.helpers import bettertitle, validated_viewname
 from nautobot.utilities.utils import (
+    convert_querydict_to_factory_formset_acceptable_querydict,
     csv_format,
     get_route_for_model,
+    get_filterable_params_from_filter_params,
     normalize_querydict,
     prepare_cloned_fields,
 )
@@ -86,7 +90,7 @@ class ObjectView(ObjectPermissionRequiredMixin, View):
             "active_tab": request.GET.get("tab", "main"),
         }
 
-    # TODO: Remove this method in 2.0. Can be retrieved from instance itself now
+    # 2.0 TODO: Remove this method in 2.0. Can be retrieved from instance itself now
     # instance.get_changelog_url()
     # Only available on models that support changelogs
     def get_changelog_url(self, instance):
@@ -131,7 +135,7 @@ class ObjectView(ObjectPermissionRequiredMixin, View):
                 "object": instance,
                 "verbose_name": self.queryset.model._meta.verbose_name,
                 "verbose_name_plural": self.queryset.model._meta.verbose_name_plural,
-                "changelog_url": changelog_url,  # TODO: Remove in 2.0. This information can be retrieved from the object itself now.
+                "changelog_url": changelog_url,  # 2.0 TODO: Remove in 2.0. This information can be retrieved from the object itself now.
                 **self.get_extra_context(request, instance),
             },
         )
@@ -166,9 +170,7 @@ class ObjectListView(ObjectPermissionRequiredMixin, View):
     def get_filter_params(self, request):
         """Helper function - take request.GET and discard any parameters that are not used for queryset filtering."""
         filter_params = request.GET.copy()
-        for non_filter_param in self.non_filter_params:
-            filter_params.pop(non_filter_param, None)
-        return filter_params
+        return get_filterable_params_from_filter_params(filter_params, self.non_filter_params, self.filterset)
 
     def get_required_permission(self):
         return get_permission_for_model(self.queryset.model, "view")
@@ -235,8 +237,12 @@ class ObjectListView(ObjectPermissionRequiredMixin, View):
         model = self.queryset.model
         content_type = ContentType.objects.get_for_model(model)
 
-        filter_params = self.get_filter_params(request)
+        display_filter_params = []
+        dynamic_filter_form = None
+        filter_form = None
+
         if self.filterset:
+            filter_params = self.get_filter_params(request)
             filterset = self.filterset(filter_params, self.queryset)
             self.queryset = filterset.qs
             if not filterset.is_valid():
@@ -246,6 +252,22 @@ class ObjectListView(ObjectPermissionRequiredMixin, View):
                 )
                 self.queryset = self.queryset.none()
 
+            display_filter_params = [
+                [field_name, values if isinstance(values, (list, tuple)) else [values]]
+                for field_name, values in filter_params.items()
+            ]
+
+            if request.GET:
+                factory_formset_params = convert_querydict_to_factory_formset_acceptable_querydict(
+                    request.GET, self.filterset
+                )
+                dynamic_filter_form = DynamicFilterFormSet(filterset_class=self.filterset, data=factory_formset_params)
+            else:
+                dynamic_filter_form = DynamicFilterFormSet(filterset_class=self.filterset)
+
+            if self.filterset_form:
+                filter_form = self.filterset_form(filter_params, label_suffix="")
+
         # Check for export template rendering
         if request.GET.get("export"):
             et = get_object_or_404(
@@ -284,26 +306,25 @@ class ObjectListView(ObjectPermissionRequiredMixin, View):
             perm_name = get_permission_for_model(model, action)
             permissions[action] = request.user.has_perm(perm_name)
 
-        # Construct the objects table
-        table = self.table(self.queryset, user=request.user)
-        if "pk" in table.base_columns and (permissions["change"] or permissions["delete"]):
-            table.columns.show("pk")
+        table = None
+        table_config_form = None
+        if self.table:
+            # Construct the objects table
+            table = self.table(self.queryset, user=request.user)
+            if "pk" in table.base_columns and (permissions["change"] or permissions["delete"]):
+                table.columns.show("pk")
 
-        # Apply the request context
-        paginate = {
-            "paginator_class": EnhancedPaginator,
-            "per_page": get_paginate_count(request),
-        }
-        RequestConfig(request, paginate).configure(table)
+            # Apply the request context
+            paginate = {
+                "paginator_class": EnhancedPaginator,
+                "per_page": get_paginate_count(request),
+            }
+            RequestConfig(request, paginate).configure(table)
+            table_config_form = TableConfigForm(table=table)
 
-        filter_form = None
-        if self.filterset_form:
-            if request.GET:
-                # Bind form to the values specified in request.GET
-                filter_form = self.filterset_form(filter_params, label_suffix="")
-            else:
-                # Use unbound form with default (initial) values
-                filter_form = self.filterset_form(label_suffix="")
+        # For the search form field, use a custom placeholder.
+        q_placeholder = "Search " + bettertitle(model._meta.verbose_name_plural)
+        search_form = SearchForm(data=request.GET, q_placeholder=q_placeholder)
 
         valid_actions = self.validate_action_buttons(request)
 
@@ -312,9 +333,20 @@ class ObjectListView(ObjectPermissionRequiredMixin, View):
             "table": table,
             "permissions": permissions,
             "action_buttons": valid_actions,
-            "table_config_form": TableConfigForm(table=table),
+            "table_config_form": table_config_form,
+            "filter_params": display_filter_params,
             "filter_form": filter_form,
+            "dynamic_filter_form": dynamic_filter_form,
+            "search_form": search_form,
+            "list_url": validated_viewname(model, "list"),
+            "title": bettertitle(model._meta.verbose_name_plural),
         }
+
+        # `extra_context()` would require `request` access, however `request` parameter cannot simply be
+        # added to `extra_context()` because  this method has been used by multiple apps without any parameters.
+        # Changing 'def extra context()' to 'def extra context(request)' might break current methods
+        # in plugins and core that either override or implement it without request.
+        setattr(self, "request", request)
         context.update(self.extra_context())
 
         return render(request, self.template_name, context)
@@ -949,8 +981,11 @@ class BulkEditView(GetReturnURLMixin, ObjectPermissionRequiredMixin, View):
         model = self.queryset.model
 
         # If we are editing *all* objects in the queryset, replace the PK list with all matched objects.
-        if request.POST.get("_all") and self.filterset is not None:
-            pk_list = [obj.pk for obj in self.filterset(request.GET, self.queryset.only("pk")).qs]
+        if request.POST.get("_all"):
+            if self.filterset is not None:
+                pk_list = [obj.pk for obj in self.filterset(request.GET, model.objects.only("pk")).qs]
+            else:
+                pk_list = model.objects.values_list("pk", flat=True)
         else:
             pk_list = request.POST.getlist("pk")
 
@@ -1010,7 +1045,7 @@ class BulkEditView(GetReturnURLMixin, ObjectPermissionRequiredMixin, View):
                                 # 2.0 TODO: #824 when we use slug in obj.cf we can just do obj.cf[field_name[3:]]
                                 if field_name in form.nullable_fields and field_name in nullified_fields:
                                     obj.cf[form_cf_to_key[field_name]] = None
-                                elif form.cleaned_data.get(field_name) not in (None, ""):
+                                elif form.cleaned_data.get(field_name) not in (None, "", []):
                                     obj.cf[form_cf_to_key[field_name]] = form.cleaned_data[field_name]
 
                             obj.full_clean()
