@@ -16,6 +16,7 @@ from nautobot.dcim.choices import (
     CableLengthUnitChoices,
     ConsolePortTypeChoices,
     DeviceFaceChoices,
+    DeviceRedundancyGroupFailoverStrategyChoices,
     InterfaceModeChoices,
     InterfaceStatusChoices,
     InterfaceTypeChoices,
@@ -43,8 +44,9 @@ from nautobot.dcim.models import (
     Device,
     DeviceBay,
     DeviceBayTemplate,
-    DeviceType,
+    DeviceRedundancyGroup,
     DeviceRole,
+    DeviceType,
     FrontPort,
     FrontPortTemplate,
     Interface,
@@ -99,6 +101,7 @@ from .nested_serializers import (  # noqa: F401
     NestedConsoleServerPortTemplateSerializer,
     NestedDeviceBaySerializer,
     NestedDeviceBayTemplateSerializer,
+    NestedDeviceRedundancyGroupSerializer,
     NestedDeviceRoleSerializer,
     NestedDeviceSerializer,
     NestedDeviceTypeSerializer,
@@ -273,6 +276,7 @@ class LocationTypeSerializer(NautobotModelSerializer):
             "name",
             "slug",
             "parent",
+            "nestable",
             "content_types",
             "description",
             "tree_depth",
@@ -348,12 +352,12 @@ class RackGroupSerializer(NautobotModelSerializer):
         ]
         # Omit the UniqueTogetherValidator that would be automatically added to validate (site, slug). This
         # prevents slug from being interpreted as a required field.
-        # TODO: Remove if/when slug is globally unique. This would be a breaking change.
+        # 2.0 TODO: Remove if/when slug is globally unique. This would be a breaking change.
         validators = [UniqueTogetherValidator(queryset=RackGroup.objects.all(), fields=("site", "name"))]
 
     def validate(self, data):
         # Validate uniqueness of (site, slug) since we omitted the automatically-created validator from Meta.
-        # TODO: Remove if/when slug is globally unique. This would be a breaking change.
+        # 2.0 TODO: Remove if/when slug is globally unique. This would be a breaking change.
         if data.get("slug", None):
             validator = UniqueTogetherValidator(queryset=RackGroup.objects.all(), fields=("site", "slug"))
             validator(data, self)
@@ -533,12 +537,12 @@ class DeviceTypeSerializer(NautobotModelSerializer, TaggedObjectSerializer):
         ]
         # Omit the UniqueTogetherValidator that would be automatically added to validate (manufacturer, slug). This
         # prevents slug from being interpreted as a required field.
-        # TODO: Remove if/when slug is globally unique. This would be a breaking change.
+        # 2.0 TODO: Remove if/when slug is globally unique. This would be a breaking change.
         validators = [UniqueTogetherValidator(queryset=DeviceType.objects.all(), fields=("manufacturer", "model"))]
 
     def validate(self, data):
         # Validate uniqueness of (manufacturer, slug) since we omitted the automatically-created validator from Meta.
-        # TODO: Remove if/when slug is globally unique. This would be a breaking change.
+        # 2.0 TODO: Remove if/when slug is globally unique. This would be a breaking change.
         if data.get("slug", None):
             validator = UniqueTogetherValidator(queryset=DeviceType.objects.all(), fields=("manufacturer", "slug"))
             validator(data, self)
@@ -756,6 +760,7 @@ class DeviceSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusMo
     secrets_group = NestedSecretsGroupSerializer(required=False, allow_null=True)
     cluster = NestedClusterSerializer(required=False, allow_null=True)
     virtual_chassis = NestedVirtualChassisSerializer(required=False, allow_null=True)
+    device_redundancy_group = NestedDeviceRedundancyGroupSerializer(required=False, allow_null=True)
     local_context_schema = NestedConfigContextSchemaSerializer(required=False, allow_null=True)
 
     class Meta:
@@ -784,6 +789,8 @@ class DeviceSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusMo
             "virtual_chassis",
             "vc_position",
             "vc_priority",
+            "device_redundancy_group",
+            "device_redundancy_group_priority",
             "comments",
             "local_context_schema",
             "local_context_data",
@@ -955,7 +962,7 @@ class PowerPortSerializer(
         ]
 
 
-# TODO: collapse this with InterfaceSerializer in 2.0.
+# 2.0 TODO: This becomes non-default in 2.0, removed in 2.2.
 class InterfaceSerializerVersion12(
     NautobotModelSerializer,
     TaggedObjectSerializer,
@@ -1025,6 +1032,16 @@ class InterfaceSerializerVersion12(
                 )
 
         # Validate many-to-many VLAN assignments
+        mode = data.get("mode", getattr(self.instance, "mode", None))
+        has_tagged_vlans = data.get("tagged_vlans", False)
+        if not has_tagged_vlans and self.instance and self.instance.tagged_vlans.exists():
+            has_tagged_vlans = True
+
+        if has_tagged_vlans and mode != InterfaceModeChoices.MODE_TAGGED:
+            raise serializers.ValidationError(
+                {"tagged_vlans": f"Mode must be set to {InterfaceModeChoices.MODE_TAGGED} when specifying tagged_vlans"}
+            )
+
         device = self.instance.device if self.instance else data.get("device")
         for vlan in data.get("tagged_vlans", []):
             if vlan.site not in [device.site, None]:
@@ -1120,6 +1137,23 @@ class DeviceBaySerializer(NautobotModelSerializer, TaggedObjectSerializer):
         ]
 
 
+class DeviceRedundancyGroupSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusModelSerializerMixin):
+    url = serializers.HyperlinkedIdentityField(view_name="dcim-api:deviceredundancygroup-detail")
+    failover_strategy = ChoiceField(choices=DeviceRedundancyGroupFailoverStrategyChoices)
+
+    class Meta:
+        model = DeviceRedundancyGroup
+        fields = [
+            "url",
+            "name",
+            "slug",
+            "description",
+            "failover_strategy",
+            "secrets_group",
+            "comments",
+        ]
+
+
 #
 # Inventory items
 #
@@ -1188,7 +1222,7 @@ class CableSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusMod
         """
         if side.lower() not in ["a", "b"]:
             raise ValueError("Termination side must be either A or B.")
-        termination = getattr(obj, "termination_{}".format(side.lower()))
+        termination = getattr(obj, f"termination_{side.lower()}")
         if termination is None:
             return None
         serializer = get_serializer_for_model(termination, prefix="Nested")
