@@ -1,3 +1,5 @@
+from decimal import Decimal
+
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.test import TestCase
@@ -7,6 +9,7 @@ from nautobot.dcim.choices import (
     CableStatusChoices,
     CableTypeChoices,
     DeviceFaceChoices,
+    InterfaceModeChoices,
     InterfaceTypeChoices,
     PortTypeChoices,
     PowerOutletFeedLegChoices,
@@ -20,6 +23,7 @@ from nautobot.dcim.models import (
     Device,
     DeviceBay,
     DeviceBayTemplate,
+    DeviceRedundancyGroup,
     DeviceRole,
     DeviceType,
     FrontPort,
@@ -42,12 +46,13 @@ from nautobot.dcim.models import (
 )
 from nautobot.extras.choices import CustomFieldTypeChoices
 from nautobot.extras.models import CustomField, Status
+from nautobot.ipam.models import VLAN
 from nautobot.tenancy.models import Tenant
 
 
 class CableLengthTestCase(TestCase):
     def setUp(self):
-        self.site = Site.objects.create(name="Test Site 1", slug="test-site-1")
+        self.site = Site.objects.first()
         self.manufacturer = Manufacturer.objects.create(name="Test Manufacturer 1", slug="test-manufacturer-1")
         self.devicetype = DeviceType.objects.create(
             manufacturer=self.manufacturer,
@@ -105,7 +110,7 @@ class InterfaceTemplateCustomFieldTestCase(TestCase):
         Check that all _custom_field_data is present and all customfields are filled with the correct default values.
         """
         statuses = Status.objects.get_for_model(Device)
-        site = Site.objects.create(name="Site 1", slug="site-1")
+        site = Site.objects.first()
         manufacturer = Manufacturer.objects.create(name="Acme", slug="acme")
         device_role = DeviceRole.objects.create(name="Device Role 1", slug="device-role-1", color="ff0000")
         custom_fields = [
@@ -147,6 +152,49 @@ class InterfaceTemplateCustomFieldTestCase(TestCase):
         self.assertEqual(Interface.objects.get(pk=interfaces[1].pk).cf["field_3"], "value_3")
 
 
+class InterfaceTemplateTestCase(TestCase):
+    def test_interface_template_sets_interface_status(self):
+        """
+        When a device is created with a device type associated with the template,
+        assert interface templates sets the interface status.
+        """
+        statuses = Status.objects.get_for_model(Device)
+        site = Site.objects.create(name="Site 1", slug="site-1")
+        manufacturer = Manufacturer.objects.create(name="Acme", slug="acme")
+        device_role = DeviceRole.objects.create(name="Device Role 1", slug="device-role-1", color="ff0000")
+        device_type = DeviceType.objects.create(manufacturer=manufacturer, model="FrameForwarder 2048", slug="ff2048")
+        InterfaceTemplate.objects.create(
+            device_type=device_type,
+            name="Test_Template_1",
+            type=InterfaceTypeChoices.TYPE_1GE_FIXED,
+            mgmt_only=True,
+        )
+        device_1 = Device.objects.create(
+            device_type=device_type,
+            device_role=device_role,
+            status=statuses[0],
+            name="Test Device 1",
+            site=site,
+        )
+
+        active_status = Status.objects.get(slug="active")
+        self.assertEqual(device_1.interfaces.get(name="Test_Template_1").status, active_status)
+
+        # Assert that a different status is picked if active status is not found for interface
+        interface_ct = ContentType.objects.get_for_model(Interface)
+        active_status.content_types.remove(interface_ct)
+
+        device_2 = Device.objects.create(
+            device_type=device_type,
+            device_role=device_role,
+            status=statuses[0],
+            name="Test Device 2",
+            site=site,
+        )
+        first_status = Status.objects.get_for_model(Interface).first()
+        self.assertIsNotNone(device_2.interfaces.get(name="Test_Template_1").status, first_status)
+
+
 class RackGroupTestCase(TestCase):
     def setUp(self):
         """
@@ -157,17 +205,14 @@ class RackGroupTestCase(TestCase):
             - Rack 1
             - PowerPanel 1
         """
-        self.site_a = Site.objects.create(name="Site A", slug="site-a")
-
-        self.location_type_a = LocationType.objects.create(name="Location Type A")
+        self.location_type_a = LocationType.objects.get(name="Campus")
         self.location_type_a.content_types.add(
             ContentType.objects.get_for_model(RackGroup),
             ContentType.objects.get_for_model(Rack),
             ContentType.objects.get_for_model(PowerPanel),
         )
-        self.location_a = Location.objects.create(
-            name="Location A", location_type=self.location_type_a, site=self.site_a
-        )
+        self.location_a = Location.objects.filter(location_type=self.location_type_a)[0]
+        self.site_a = self.location_a.site
 
         self.rackgroup_a1 = RackGroup(
             site=self.site_a, location=self.location_a, name="RackGroup A1", slug="rackgroup-a1"
@@ -192,7 +237,7 @@ class RackGroupTestCase(TestCase):
     def test_rackgroup_location_validation(self):
         """Check that rack group locations are validated correctly."""
         # Child group cannot belong to a different site than its parent
-        site_b = Site.objects.create(name="Site B", slug="site-b")
+        site_b = Site.objects.exclude(locations__in=[self.location_a]).first()
         child = RackGroup(site=site_b, parent=self.rackgroup_a1, name="Child Group")
         with self.assertRaises(ValidationError) as cm:
             child.validated_save()
@@ -203,25 +248,25 @@ class RackGroupTestCase(TestCase):
         child = RackGroup(site=self.site_a, parent=self.rackgroup_a1, location=location_b, name="Child Group")
         with self.assertRaises(ValidationError) as cm:
             child.validated_save()
-        self.assertIn('Location "Location B" does not belong to site "Site A"', str(cm.exception))
+        self.assertIn(f'Location "{location_b.name}" does not belong to site "{self.site_a.name}"', str(cm.exception))
 
         # Group location, if specified, must permit RackGroups
-        location_type_c = LocationType.objects.create(name="Location Type C")
+        location_type_c = LocationType.objects.get(name="Elevator")
         location_c = Location.objects.create(name="Location C", location_type=location_type_c, site=self.site_a)
         child = RackGroup(site=self.site_a, parent=self.rackgroup_a1, location=location_c, name="Child Group")
         with self.assertRaises(ValidationError) as cm:
             child.validated_save()
-        self.assertIn('Rack groups may not associate to locations of type "Location Type C"', str(cm.exception))
+        self.assertIn(f'Rack groups may not associate to locations of type "{location_type_c}"', str(cm.exception))
 
         # Child group location must descend from parent group location
-        location_type_d = LocationType.objects.create(name="Location Type D", parent=location_type_c)
+        location_type_d = LocationType.objects.get(name="Room")
         location_type_d.content_types.add(ContentType.objects.get_for_model(RackGroup))
         location_d = Location.objects.create(name="Location D", location_type=location_type_d, parent=location_c)
         child = RackGroup(site=self.site_a, parent=self.rackgroup_a1, location=location_d, name="Child Group")
         with self.assertRaises(ValidationError) as cm:
             child.validated_save()
         self.assertIn(
-            'Location "Location D" is not descended from parent rack group "RackGroup A1" location "Location A"',
+            f'Location "Location D" is not descended from parent rack group "RackGroup A1" location "{self.location_a.name}"',
             str(cm.exception),
         )
 
@@ -229,7 +274,8 @@ class RackGroupTestCase(TestCase):
         """
         Check that all child RackGroups, Racks, and PowerPanels get updated when a RackGroup is moved to a new Site.
         """
-        site_b = Site.objects.create(name="Site B", slug="site-b")
+        existing_rackgroup_site = self.rackgroup_a1.site
+        site_b = Site.objects.exclude(pk=existing_rackgroup_site.pk).last()
 
         # Move RackGroup A1 to Site B
         self.rackgroup_a1.site = site_b
@@ -296,9 +342,9 @@ class RackTestCase(TestCase):
             ContentType.objects.get_for_model(Device),
         )
 
-        self.site1 = Site.objects.create(name="TestSite1", slug="test-site-1")
+        self.site1 = Site.objects.first()
         self.location1 = Location.objects.create(name="Location1", location_type=self.location_type_a, site=self.site1)
-        self.site2 = Site.objects.create(name="TestSite2", slug="test-site-2")
+        self.site2 = Site.objects.all()[1]
         self.group1 = RackGroup.objects.create(
             name="TestGroup1", slug="test-group-1", site=self.site1, location=self.location1
         )
@@ -432,9 +478,9 @@ class RackTestCase(TestCase):
         """
         Check that child Devices get updated when a Rack is moved to a new Site.
         """
-        site_a = Site.objects.create(name="Site A", slug="site-a")
+        site_a = Site.objects.first()
         location_a = Location.objects.create(name="Location A", location_type=self.location_type_a, site=site_a)
-        site_b = Site.objects.create(name="Site B", slug="site-b")
+        site_b = Site.objects.all()[1]
         location_b = Location.objects.create(name="Location B", location_type=self.location_type_a, site=site_b)
 
         manufacturer = Manufacturer.objects.create(name="Manufacturer 1", slug="manufacturer-1")
@@ -522,7 +568,7 @@ class RackTestCase(TestCase):
         rack = Rack(name="Rack", site=self.site2, location=self.location1, status=self.status)
         with self.assertRaises(ValidationError) as cm:
             rack.validated_save()
-        self.assertIn('Location "Location1" does not belong to site "TestSite2"', str(cm.exception))
+        self.assertIn(f'Location "Location1" does not belong to site "{self.site2.name}"', str(cm.exception))
 
         # Rack group location and rack location must relate
         location2 = Location.objects.create(name="Location2", location_type=self.location_type_a, site=self.site1)
@@ -558,15 +604,50 @@ class LocationTypeTestCase(TestCase):
                 LocationType(name=candidate_name).clean()
             self.assertIn("This name is reserved", str(cm.exception))
 
+    def test_changing_parent(self):
+        """Validate clean logic around changing the parent of a LocationType."""
+        parent = LocationType.objects.create(name="Parent LocationType")
+        child = LocationType.objects.create(name="Child LocationType")
+
+        # If there are no Locations using it yet, parent can be freely changed
+        child.parent = None
+        child.validated_save()
+        child.parent = parent
+        child.validated_save()
+
+        # Once there are Locations using it, parent cannot be changed.
+        site = Site.objects.create(name="Test Site", status=Status.objects.get_for_model(Site).first())
+        parent_loc = Location.objects.create(
+            name="Parent 1", location_type=parent, site=site, status=Status.objects.get_for_model(Location).first()
+        )
+        child_loc = Location.objects.create(
+            name="Child 1", location_type=child, parent=parent_loc, status=Status.objects.get_for_model(Location).last()
+        )
+        child.parent = None
+        with self.assertRaisesMessage(
+            ValidationError,
+            "This LocationType currently has Locations using it, therefore its parent cannot be changed at this time.",
+        ):
+            child.validated_save()
+
+        # If the locations are deleted, it again becomes re-parent-able.
+        child_loc.delete()
+        child.validated_save()
+
 
 class LocationTestCase(TestCase):
     def setUp(self):
-        self.root_type = LocationType.objects.create(name="Campus")
-        self.intermediate_type = LocationType.objects.create(name="Building", parent=self.root_type)
-        self.leaf_type = LocationType.objects.create(name="Room", parent=self.intermediate_type)
+        self.root_type = LocationType.objects.get(name="Campus")
+        self.intermediate_type = LocationType.objects.get(name="Building")
+        self.leaf_type = LocationType.objects.get(name="Floor")
+
+        self.root_nestable_type = LocationType.objects.get(name="Root")
+        self.leaf_nestable_type = LocationType.objects.create(
+            name="Pseudo-RackGroup", parent=self.root_nestable_type, nestable=True
+        )
 
         self.status = Status.objects.get(slug="active")
-        self.site = Site.objects.create(name="Test Site", status=self.status)
+        self.site = Site.objects.first()
 
     def test_validate_unique(self):
         """Confirm that the uniqueness constraint on (parent, name) works when parent is None."""
@@ -581,7 +662,7 @@ class LocationTestCase(TestCase):
         """Once created, a location cannot change location_type."""
         location = Location(name="Campus 1", location_type=self.root_type, site=self.site, status=self.status)
         location.validated_save()
-        location.location_type = self.intermediate_type
+        location.location_type = self.root_nestable_type
         with self.assertRaises(ValidationError) as cm:
             location.validated_save()
         self.assertIn("location_type", str(cm.exception))
@@ -589,19 +670,70 @@ class LocationTestCase(TestCase):
 
     def test_parent_type_must_match(self):
         """A location's parent's location_type must match its location_type's parent."""
-        location_1 = Location(name="Campus 1", location_type=self.root_type, site=self.site, status=self.status)
+        location_1 = Location(name="Building 1", location_type=self.root_type, site=self.site, status=self.status)
         location_1.validated_save()
         location_2 = Location(name="Room 1", location_type=self.leaf_type, parent=location_1, status=self.status)
         with self.assertRaises(ValidationError) as cm:
             location_2.validated_save()
-        self.assertIn("must have a parent Location of type Building", str(cm.exception))
+        self.assertIn(
+            "A Location of type Floor can only have a Location of type Building as its parent.", str(cm.exception)
+        )
+
+    def test_parent_type_nestable_logic(self):
+        """A location of a nestable type may have a parent of the same type."""
+        # A location using a root-level nestable type can have a site rather than a parent
+        location_1 = Location(
+            name="Region 1", location_type=self.root_nestable_type, site=self.site, status=self.status
+        )
+        location_1.validated_save()
+        # A location using a root-level nestable type can have a parent rather than a site
+        location_2 = Location(
+            name="Region 1-A", location_type=self.root_nestable_type, parent=location_1, status=self.status
+        )
+        location_2.validated_save()
+        # A location can't have both a parent and a site, even if it's a "root" location-type.
+        location_2.site = self.site
+        with self.assertRaises(ValidationError) as cm:
+            location_2.validated_save()
+        self.assertIn("cannot have both a parent Location and an associated Site", str(cm.exception))
+        # A location using a lower-level nestable type can be parented under the parent location type
+        location_3 = Location(
+            name="RackGroup 3", location_type=self.leaf_nestable_type, parent=location_2, status=self.status
+        )
+        location_3.validated_save()
+        # A location using a lower-level nestable type can be parented under its own type
+        location_4 = Location(
+            name="RackGroup 3-B", location_type=self.leaf_nestable_type, parent=location_3, status=self.status
+        )
+        location_4.validated_save()
+        # Can't mix and match location types though
+        with self.assertRaises(ValidationError) as cm:
+            location_5 = Location(
+                name="Region 5", location_type=self.root_nestable_type, parent=location_4, status=self.status
+            )
+            location_5.validated_save()
+        self.assertIn("only have a Location of the same type as its parent", str(cm.exception))
+        location_6 = Location(name="Campus 1", location_type=self.root_type, site=self.site, status=self.status)
+        location_6.validated_save()
+        with self.assertRaises(ValidationError) as cm:
+            location_7 = Location(
+                name="RackGroup 7",
+                location_type=self.leaf_nestable_type,
+                parent=location_6,
+                status=self.status,
+            )
+            location_7.validated_save()
+        self.assertIn(
+            f"only have a Location of the same type or of type {self.root_nestable_type} as its parent",
+            str(cm.exception),
+        )
 
     def test_site_required_for_root(self):
         """A Location of a root type must have a Site."""
         location = Location(name="Campus 1", location_type=self.root_type, status=self.status)
         with self.assertRaises(ValidationError) as cm:
             location.validated_save()
-        self.assertIn("must have a Site", str(cm.exception))
+        self.assertIn("must have an associated Site", str(cm.exception))
 
     def test_site_forbidden_for_non_root(self):
         """A Location of a non-root type must have a parent, not a Site."""
@@ -622,7 +754,7 @@ class LocationTestCase(TestCase):
 class DeviceTestCase(TestCase):
     def setUp(self):
 
-        self.site = Site.objects.create(name="Test Site 1", slug="test-site-1")
+        self.site = Site.objects.first()
         manufacturer = Manufacturer.objects.create(name="Test Manufacturer 1", slug="test-manufacturer-1")
         self.device_type = DeviceType.objects.create(
             manufacturer=manufacturer,
@@ -633,8 +765,8 @@ class DeviceTestCase(TestCase):
             name="Test Device Role 1", slug="test-device-role-1", color="ff0000"
         )
         self.device_status = Status.objects.get_for_model(Device).get(slug="active")
-        self.location_type_1 = LocationType.objects.create(name="Root Type")
-        self.location_type_2 = LocationType.objects.create(name="Leaf Type", parent=self.location_type_1)
+        self.location_type_1 = LocationType.objects.get(name="Building")
+        self.location_type_2 = LocationType.objects.get(name="Floor")
         self.location_type_2.content_types.add(ContentType.objects.get_for_model(Device))
         self.location_1 = Location.objects.create(
             name="Root", status=self.device_status, location_type=self.location_type_1, site=self.site
@@ -642,6 +774,7 @@ class DeviceTestCase(TestCase):
         self.location_2 = Location.objects.create(
             name="Leaf", status=self.device_status, location_type=self.location_type_2, parent=self.location_1
         )
+        self.device_redundancy_group = DeviceRedundancyGroup.objects.first()
 
         # Create DeviceType components
         ConsolePortTemplate(device_type=self.device_type, name="Console Port 1").save()
@@ -794,7 +927,7 @@ class DeviceTestCase(TestCase):
         device2.save()
 
     def test_device_location_site_mismatch(self):
-        other_site = Site.objects.create(name="Test Site 2", status=self.device_status)
+        other_site = Site.objects.all()[1]
         device = Device(
             name="Device 3",
             device_type=self.device_type,
@@ -805,7 +938,7 @@ class DeviceTestCase(TestCase):
         )
         with self.assertRaises(ValidationError) as cm:
             device.validated_save()
-        self.assertIn('Location "Leaf" does not belong to site "Test Site 2"', str(cm.exception))
+        self.assertIn(f'Location "Leaf" does not belong to site "{other_site.name}"', str(cm.exception))
 
     def test_device_location_content_type_not_allowed(self):
         device = Device(
@@ -818,13 +951,61 @@ class DeviceTestCase(TestCase):
         )
         with self.assertRaises(ValidationError) as cm:
             device.validated_save()
-        self.assertIn('Devices may not associate to locations of type "Root Type"', str(cm.exception))
+        self.assertIn(
+            f'Devices may not associate to locations of type "{self.location_type_1.name}"', str(cm.exception)
+        )
+
+    def test_device_redundancy_group_validation(self):
+        d1 = Device(
+            name="Test Device 1",
+            device_type=self.device_type,
+            device_role=self.device_role,
+            status=self.device_status,
+            site=self.site,
+        )
+        d1.validated_save()
+
+        d2 = Device(
+            name="Test Device 2",
+            device_type=self.device_type,
+            device_role=self.device_role,
+            status=self.device_status,
+            site=self.site,
+        )
+        d2.validated_save()
+
+        # Validate we can set a redundancy group without any priority set
+        d1.device_redundancy_group = self.device_redundancy_group
+        d1.validated_save()
+
+        # Validate two devices can be a part of the same redundancy group without any priority set
+        d2.device_redundancy_group = self.device_redundancy_group
+        d2.validated_save()
+
+        # Validate we can assign a priority to at least one device in the group
+        d1.device_redundancy_group_priority = 1
+        d1.validated_save()
+
+        # Validate both devices in the same group can have the same priority
+        d2.device_redundancy_group_priority = 1
+        d2.validated_save()
+
+        # Validate devices in the same group can have different priority
+        d2.device_redundancy_group_priority = 2
+        d2.validated_save()
+
+        # Validate devices cannot have an assigned priority without an assigned group
+        d1.device_redundancy_group = None
+        with self.assertRaisesMessage(
+            ValidationError, "Must assign a redundancy group when defining a redundancy group priority."
+        ):
+            d1.validated_save()
 
 
 class CableTestCase(TestCase):
     def setUp(self):
 
-        site = Site.objects.create(name="Test Site 1", slug="test-site-1")
+        site = Site.objects.first()
         manufacturer = Manufacturer.objects.create(name="Test Manufacturer 1", slug="test-manufacturer-1")
         devicetype = DeviceType.objects.create(
             manufacturer=manufacturer,
@@ -1084,30 +1265,32 @@ class CableTestCase(TestCase):
 class PowerPanelTestCase(TestCase):
     def test_power_panel_validation(self):
         active = Status.objects.get(name="Active")
-        site_1 = Site.objects.create(name="Site 1", status=active)
+        site_1 = Site.objects.first()
+        site_1.status = active
         location_type_1 = LocationType.objects.create(name="Location Type 1")
         location_1 = Location.objects.create(
             name="Location 1", location_type=location_type_1, site=site_1, status=active
         )
-
         power_panel = PowerPanel(name="Power Panel 1", site=site_1, location=location_1)
         with self.assertRaises(ValidationError) as cm:
             power_panel.validated_save()
-        self.assertIn('Power panels may not associate to locations of type "Location Type 1"', str(cm.exception))
+        self.assertIn(f'Power panels may not associate to locations of type "{location_type_1}"', str(cm.exception))
 
         location_type_1.content_types.add(ContentType.objects.get_for_model(PowerPanel))
-        site_2 = Site.objects.create(name="Site 2", status=active)
+        site_2 = Site.objects.all()[1]
         power_panel.site = site_2
         with self.assertRaises(ValidationError) as cm:
             power_panel.validated_save()
-        self.assertIn('Location "Location 1" does not belong to site "Site 2"', str(cm.exception))
+        self.assertIn(f'Location "{location_1.name}" does not belong to site "{site_2.name}"', str(cm.exception))
 
         power_panel.site = site_1
         rack_group = RackGroup.objects.create(name="Rack Group 1", site=site_2)
         power_panel.rack_group = rack_group
         with self.assertRaises(ValidationError) as cm:
             power_panel.validated_save()
-        self.assertIn("Rack group Rack Group 1 (Site 2) is in a different site than Site 1", str(cm.exception))
+        self.assertIn(
+            f"Rack group Rack Group 1 ({site_2.name}) is in a different site than {site_1.name}", str(cm.exception)
+        )
 
         rack_group.site = site_1
         location_2 = Location.objects.create(
@@ -1118,6 +1301,66 @@ class PowerPanelTestCase(TestCase):
         with self.assertRaises(ValidationError) as cm:
             power_panel.validated_save()
         self.assertIn(
-            'Rack group "Rack Group 1" belongs to a location ("Location 2") that does not contain "Location 1"',
+            f'Rack group "Rack Group 1" belongs to a location ("{location_2.name}") that does not contain "{location_1.name}"',
             str(cm.exception),
         )
+
+
+class InterfaceTestCase(TestCase):
+    def setUp(self):
+        manufacturer = Manufacturer.objects.create(name="Manufacturer 1", slug="manufacturer-1")
+        devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type 1", slug="device-type-1")
+        devicerole = DeviceRole.objects.create(name="Device Role 1", slug="device-role-1")
+        site = Site.objects.create(name="Site-1", slug="site-1")
+        self.vlan = VLAN.objects.create(name="VLAN 1", vid=100, site=site)
+        status = Status.objects.get_for_model(Device)[0]
+        self.device = Device.objects.create(
+            name="Device 1",
+            device_type=devicetype,
+            device_role=devicerole,
+            site=site,
+            status=status,
+        )
+
+    def test_tagged_vlan_raise_error_if_mode_not_set_to_tagged(self):
+        interface = Interface.objects.create(
+            name="Int1",
+            type=InterfaceTypeChoices.TYPE_VIRTUAL,
+            device=self.device,
+        )
+        with self.assertRaises(ValidationError) as err:
+            interface.tagged_vlans.add(self.vlan)
+        self.assertEqual(
+            err.exception.message_dict["tagged_vlans"][0], "Mode must be set to tagged when specifying tagged_vlans"
+        )
+
+    def test_tagged_vlan_raise_error_if_mode_is_changed_without_clearing_tagged_vlans(self):
+        interface = Interface.objects.create(
+            name="Int2",
+            type=InterfaceTypeChoices.TYPE_VIRTUAL,
+            device=self.device,
+            mode=InterfaceModeChoices.MODE_TAGGED,
+        )
+        interface.tagged_vlans.add(self.vlan)
+
+        interface.mode = InterfaceModeChoices.MODE_ACCESS
+        with self.assertRaises(ValidationError) as err:
+            interface.validated_save()
+        self.assertEqual(err.exception.message_dict["tagged_vlans"][0], "Clear tagged_vlans to set mode to access")
+
+
+class SiteTestCase(TestCase):
+    def test_latitude_or_longitude(self):
+        """Test latitude and longitude is parsed to string."""
+        active_status = Status.objects.get_for_model(Site).get(slug="active")
+        site = Site(
+            name="Site A",
+            slug="site-a",
+            status=active_status,
+            longitude=55.1234567896,
+            latitude=55.1234567896,
+        )
+        site.validated_save()
+
+        self.assertEqual(site.longitude, Decimal("55.123457"))
+        self.assertEqual(site.latitude, Decimal("55.123457"))
