@@ -1,6 +1,7 @@
 from django.urls import path
 
-from nautobot.extras.views import ObjectChangeLogView, ObjectNotesView, ImageAttachmentEditView
+from nautobot.core.views.routers import NautobotUIViewSetRouter
+from nautobot.extras.views import ObjectChangeLogView, ObjectDynamicGroupsView, ObjectNotesView, ImageAttachmentEditView
 from nautobot.ipam.views import ServiceEditView
 from . import views
 from .models import (
@@ -33,6 +34,10 @@ from .models import (
 )
 
 app_name = "dcim"
+
+router = NautobotUIViewSetRouter()
+router.register("device-redundancy-groups", views.DeviceRedundancyGroupUIViewSet)
+
 urlpatterns = [
     # Regions
     path("regions/", views.RegionListView.as_view(), name="region_list"),
@@ -287,6 +292,12 @@ urlpatterns = [
         name="rack_notes",
         kwargs={"model": Rack},
     ),
+    path(
+        "racks/<uuid:pk>/dynamic-groups/",
+        ObjectDynamicGroupsView.as_view(),
+        name="rack_dynamicgroups",
+        kwargs={"model": Rack},
+    ),
     path(
         "racks/<uuid:object_id>/images/add/",
         ImageAttachmentEditView.as_view(),
@@ -782,6 +793,12 @@ urlpatterns = [
         name="device_notes",
         kwargs={"model": Device},
     ),
+    path(
+        "devices/<uuid:pk>/dynamic-groups/",
+        views.DeviceDynamicGroupsView.as_view(),
+        name="device_dynamicgroups",
+        kwargs={"model": Device},
+    ),
     path(
         "devices/<uuid:pk>/status/",
         views.DeviceStatusView.as_view(),
@@ -1618,3 +1635,4 @@ urlpatterns = [
         kwargs={"termination_a_type": PowerFeed},
     ),
 ]
+urlpatterns += router.urls
