@@ -1,5 +1,8 @@
+import logging
+
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
+from django.urls import NoReverseMatch
 from django.utils.functional import classproperty
 from drf_spectacular.utils import extend_schema_field
 from rest_framework import serializers
@@ -113,6 +116,8 @@ from .nested_serializers import (  # noqa: F401
 # Mixins and Base Classes
 #
 
+logger = logging.getLogger(__name__)
+
 
 class NotesSerializerMixin(BaseModelSerializer):
     """Extend Serializer with a `notes` field."""
@@ -128,8 +133,22 @@ class NotesSerializerMixin(BaseModelSerializer):
 
     @extend_schema_field(serializers.URLField())
     def get_notes_url(self, instance):
-        notes_url = get_route_for_model(instance, "notes", api=True)
-        return reverse(notes_url, args=[instance.id], request=self.context["request"])
+        try:
+            notes_url = get_route_for_model(instance, "notes", api=True)
+            return reverse(notes_url, args=[instance.id], request=self.context["request"])
+        except NoReverseMatch:
+            model_name = type(instance).__name__
+            logger.warning(
+                (
+                    f"Notes feature is not available for model {model_name}. "
+                    "Please make sure to: "
+                    f"1. Include NotesMixin from nautobot.extras.model.mixins in the {model_name} class definition "
+                    f"2. Include NotesViewSetMixin from nautobot.extras.api.mixins in the {model_name}ViewSet "
+                    "before including NotesSerializerMixin in the model serializer"
+                )
+            )
+
+            return None
 
 
 class NautobotModelSerializer(
@@ -176,7 +195,7 @@ class TagSerializerField(NestedTagSerializer):
         return queryset.get_for_model(model)
 
 
-# TODO should be TaggedModelSerializerMixin
+# 2.0 TODO should be TaggedModelSerializerMixin
 class TaggedObjectSerializer(BaseModelSerializer):
     tags = TagSerializerField(many=True, required=False)
 
@@ -208,7 +227,7 @@ class TaggedObjectSerializer(BaseModelSerializer):
 
     def _save_tags(self, instance, tags):
         if tags:
-            instance.tags.set(*[t.name for t in tags])
+            instance.tags.set([t.name for t in tags])
         else:
             instance.tags.clear()
 
@@ -755,6 +774,8 @@ class JobSerializer(NautobotModelSerializer, TaggedObjectSerializer):
             "soft_time_limit_override",
             "time_limit",
             "time_limit_override",
+            "task_queues",
+            "task_queues_override",
             "tags",
         ]
 
@@ -951,6 +972,7 @@ class JobInputSerializer(serializers.Serializer):
     data = serializers.JSONField(required=False, default=dict)
     commit = serializers.BooleanField(required=False, default=None)
     schedule = NestedScheduledJobSerializer(required=False)
+    task_queue = serializers.CharField(required=False, allow_blank=True)
 
 
 class JobLogEntrySerializer(BaseModelSerializer):
@@ -1084,6 +1106,7 @@ class RelationshipSerializer(ValidatedModelSerializer, NotesSerializerMixin):
             "slug",
             "description",
             "type",
+            "required_on",
             "source_type",
             "source_label",
             "source_hidden",
@@ -1233,7 +1256,7 @@ class TagSerializer(NautobotModelSerializer):
 
         # All relevant content_types should be assigned to newly created tag for API Version <1.3
         if (self.instance is None or not self.instance.present_in_database) and "content_types" not in data:
-            data["content_types"] = TaggableClassesQuery().as_queryset
+            data["content_types"] = TaggableClassesQuery().as_queryset()
 
         # check if tag is assigned to any of the removed content_types
         if self.instance is not None and self.instance.present_in_database and "content_types" in data:
@@ -1248,7 +1271,7 @@ class TagSerializer(NautobotModelSerializer):
 
 class TagSerializerVersion13(TagSerializer):
     content_types = ContentTypeField(
-        queryset=TaggableClassesQuery().as_queryset,
+        queryset=TaggableClassesQuery().as_queryset(),
         many=True,
         required=True,
     )
