@@ -1,4 +1,5 @@
 from django import forms
+from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
@@ -7,7 +8,7 @@ from django.forms import ModelMultipleChoiceField, inlineformset_factory
 from django.urls.base import reverse
 from django.utils.safestring import mark_safe
 
-from nautobot.dcim.models import DeviceRole, DeviceType, Location, Platform, Region, Site
+from nautobot.dcim.models import DeviceRedundancyGroup, DeviceRole, DeviceType, Location, Platform, Region, Site
 from nautobot.tenancy.models import Tenant, TenantGroup
 from nautobot.utilities.deprecation import class_deprecated_in_favor_of
 from nautobot.utilities.forms import (
@@ -216,6 +217,9 @@ class ConfigContextForm(BootstrapMixin, NoteModelFormMixin, forms.ModelForm):
     clusters = DynamicModelMultipleChoiceField(queryset=Cluster.objects.all(), required=False)
     tenant_groups = DynamicModelMultipleChoiceField(queryset=TenantGroup.objects.all(), required=False)
     tenants = DynamicModelMultipleChoiceField(queryset=Tenant.objects.all(), required=False)
+    device_redundancy_groups = DynamicModelMultipleChoiceField(
+        queryset=DeviceRedundancyGroup.objects.all(), required=False
+    )
 
     data = JSONField(label="")
 
@@ -237,6 +241,7 @@ class ConfigContextForm(BootstrapMixin, NoteModelFormMixin, forms.ModelForm):
             "clusters",
             "tenant_groups",
             "tenants",
+            "device_redundancy_groups",
             "tags",
             "data",
         )
@@ -258,7 +263,6 @@ class ConfigContextBulkEditForm(BootstrapMixin, NoteModelBulkEditFormMixin, Bulk
 
 class ConfigContextFilterForm(BootstrapMixin, forms.Form):
     q = forms.CharField(required=False, label="Search")
-    # FIXME(glenn) filtering by owner_content_type
     schema = DynamicModelChoiceField(queryset=ConfigContextSchema.objects.all(), to_field_name="slug", required=False)
     region = DynamicModelMultipleChoiceField(queryset=Region.objects.all(), to_field_name="slug", required=False)
     site = DynamicModelMultipleChoiceField(queryset=Site.objects.all(), to_field_name="slug", required=False)
@@ -274,6 +278,9 @@ class ConfigContextFilterForm(BootstrapMixin, forms.Form):
         queryset=TenantGroup.objects.all(), to_field_name="slug", required=False
     )
     tenant = DynamicModelMultipleChoiceField(queryset=Tenant.objects.all(), to_field_name="slug", required=False)
+    device_redundancy_group = DynamicModelMultipleChoiceField(
+        queryset=DeviceRedundancyGroup.objects.all(), to_field_name="slug", required=False
+    )
     tag = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), to_field_name="slug", required=False)
 
 
@@ -308,7 +315,6 @@ class ConfigContextSchemaBulkEditForm(NautobotBulkEditForm):
 
 class ConfigContextSchemaFilterForm(BootstrapMixin, forms.Form):
     q = forms.CharField(required=False, label="Search")
-    # FIXME(glenn) filtering by owner_content_type
 
 
 #
@@ -352,6 +358,7 @@ class CustomFieldForm(BootstrapMixin, forms.ModelForm):
         model = CustomField
         fields = (
             "label",
+            "grouping",
             "slug",
             "type",
             "weight",
@@ -727,11 +734,18 @@ class JobForm(BootstrapMixin, forms.Form):
         label="Commit changes",
         help_text="Commit changes to the database (uncheck for a dry-run)",
     )
+    _task_queue = forms.ChoiceField(
+        required=False,
+        help_text="The task queue to route this job to",
+        label="Task queue",
+    )
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        # Move _commit to the end of the form
+        # Move _task_queue and _commit to the end of the form
+        task_queue = self.fields.pop("_task_queue")
+        self.fields["_task_queue"] = task_queue
         commit = self.fields.pop("_commit")
         self.fields["_commit"] = commit
 
@@ -769,8 +783,10 @@ class JobEditForm(NautobotModelForm):
             "soft_time_limit",
             "time_limit_override",
             "time_limit",
-            "has_sensitive_variables",
             "has_sensitive_variables_override",
+            "has_sensitive_variables",
+            "task_queues_override",
+            "task_queues",
             "tags",
         ]
 
@@ -894,6 +910,7 @@ class JobScheduleForm(BootstrapMixin, forms.Form):
         required=False,
         label="Starting date and time",
         widget=DateTimePicker(),
+        help_text=f"The scheduled time is relative to the Nautobot configured timezone: {settings.TIME_ZONE}.",
     )
     _recurrence_custom_time = forms.CharField(
         required=False,
@@ -941,7 +958,7 @@ class JobResultFilterForm(BootstrapMixin, forms.Form):
         to_field_name="slug",
         widget=APISelectMultiple(api_url="/api/extras/jobs/", api_version="1.3"),
     )
-    # FIXME(glenn) Filtering by obj_type?
+    # 2.0 TODO(glenn) filtering by obj_type should be solved by dynamic filter form generation
     name = forms.CharField(required=False)
     user = DynamicModelMultipleChoiceField(
         queryset=get_user_model().objects.all(),
@@ -1092,6 +1109,7 @@ class RelationshipForm(BootstrapMixin, forms.ModelForm):
             "slug",
             "description",
             "type",
+            "required_on",
             "advanced_ui",
             "source_type",
             "source_label",
@@ -1295,7 +1313,7 @@ class TagForm(NautobotModelForm):
     slug = SlugField()
     content_types = ModelMultipleChoiceField(
         label="Content Type(s)",
-        queryset=TaggableClassesQuery().as_queryset,
+        queryset=TaggableClassesQuery().as_queryset(),
     )
 
     class Meta:
@@ -1334,7 +1352,7 @@ class TagFilterForm(NautobotFilterForm):
         choices_as_strings=True,
         required=False,
         label="Content Type(s)",
-        queryset=TaggableClassesQuery().as_queryset,
+        queryset=TaggableClassesQuery().as_queryset(),
     )
 
 
