@@ -1,10 +1,9 @@
 from django import template
 from django.contrib.contenttypes.models import ContentType
-from django.utils.html import format_html, escape
+from django.utils.html import escape
 from django.utils.safestring import mark_safe
 
-from nautobot.extras.models import CustomLink, ComputedField
-
+from nautobot.extras.models import ComputedField
 
 register = template.Library()
 
@@ -19,17 +18,18 @@ def has_computed_fields(context, obj):
 
 
 @register.simple_tag(takes_context=True)
-def computed_fields(context, obj):
+def computed_fields(context, obj, advanced_ui=None):
     """
     Render all applicable links for the given object.
+    This can also check whether the advanced_ui attribute is True or False for UI display purposes.
     """
-    computed_fields = obj.get_computed_fields(label_as_key=True)
+    fields = obj.get_computed_fields(label_as_key=True, advanced_ui=advanced_ui)
     if not computed_fields:
         return ""
 
     template_code = ""
 
-    for label, value in computed_fields.items():
+    for label, value in fields.items():
         escaped_label = escape(label)
         template_code += f"""
             <tr>
