@@ -1,5 +1,6 @@
 from concurrent.futures.thread import ThreadPoolExecutor
 import json
+import logging
 from random import shuffle
 
 from django.db import connection
@@ -56,16 +57,6 @@ class VRFTest(APIViewTestCases.APIViewTestCase):
         "description": "New description",
     }
 
-    @classmethod
-    def setUpTestData(cls):
-
-        vrfs = (
-            VRF(name="VRF 1", rd="65000:1"),
-            VRF(name="VRF 2", rd="65000:2"),
-            VRF(name="VRF 3"),  # No RD
-        )
-        VRF.objects.bulk_create(vrfs)
-
 
 class RouteTargetTest(APIViewTestCases.APIViewTestCase):
     model = RouteTarget
@@ -85,16 +76,6 @@ class RouteTargetTest(APIViewTestCases.APIViewTestCase):
         "description": "New description",
     }
 
-    @classmethod
-    def setUpTestData(cls):
-
-        route_targets = (
-            RouteTarget(name="65000:1001"),
-            RouteTarget(name="65000:1002"),
-            RouteTarget(name="65000:1003"),
-        )
-        RouteTarget.objects.bulk_create(route_targets)
-
 
 class RIRTest(APIViewTestCases.APIViewTestCase):
     model = RIR
@@ -122,15 +103,16 @@ class RIRTest(APIViewTestCases.APIViewTestCase):
 
     slug_source = "name"
 
-    @classmethod
-    def setUpTestData(cls):
+    def get_deletable_object(self):
+        return RIR.objects.create(name="DELETE ME")
 
-        rirs = (
-            RIR(name="RIR 1", slug="rir-1"),
-            RIR(name="RIR 2", slug="rir-2"),
-            RIR(name="RIR 3", slug="rir-3"),
-        )
-        RIR.objects.bulk_create(rirs)
+    def get_deletable_object_pks(self):
+        RIRs = [
+            RIR.objects.create(name="Deletable RIR 1"),
+            RIR.objects.create(name="Deletable RIR 2"),
+            RIR.objects.create(name="Deletable RIR 3"),
+        ]
+        return [rir.pk for rir in RIRs]
 
 
 class AggregateTest(APIViewTestCases.APIViewTestCase):
@@ -143,28 +125,20 @@ class AggregateTest(APIViewTestCases.APIViewTestCase):
     @classmethod
     def setUpTestData(cls):
 
-        rirs = (
-            RIR.objects.create(name="RIR 1", slug="rir-1"),
-            RIR.objects.create(name="RIR 2", slug="rir-2"),
-        )
-
-        Aggregate.objects.create(prefix=IPNetwork("10.0.0.0/8"), rir=rirs[0])
-        Aggregate.objects.create(prefix=IPNetwork("172.16.0.0/12"), rir=rirs[0])
-        Aggregate.objects.create(prefix=IPNetwork("192.168.0.0/16"), rir=rirs[0])
-        Aggregate.objects.create(prefix=IPNetwork("2001:db8:abcd::/64"), rir=rirs[0])
+        rir = RIR.objects.filter(is_private=False).first()
 
         cls.create_data = [
             {
-                "prefix": "100.0.0.0/8",
-                "rir": rirs[1].pk,
+                "prefix": "12.0.0.0/8",
+                "rir": rir.pk,
             },
             {
-                "prefix": "2001:db8:abcd:12::/64",
-                "rir": rirs[1].pk,
+                "prefix": "2d00::/8",
+                "rir": rir.pk,
             },
             {
-                "prefix": "102.0.0.0/8",
-                "rir": rirs[1].pk,
+                "prefix": "17.0.0.0/16",
+                "rir": rir.pk,
             },
         ]
 
@@ -194,16 +168,6 @@ class RoleTest(APIViewTestCases.APIViewTestCase):
     }
     slug_source = "name"
 
-    @classmethod
-    def setUpTestData(cls):
-
-        roles = (
-            Role(name="Role 1", slug="role-1"),
-            Role(name="Role 2", slug="role-2"),
-            Role(name="Role 3", slug="role-3"),
-        )
-        Role.objects.bulk_create(roles)
-
 
 class PrefixTest(APIViewTestCases.APIViewTestCase):
     model = Prefix
@@ -228,85 +192,71 @@ class PrefixTest(APIViewTestCases.APIViewTestCase):
     }
     choices_fields = ["status"]
 
+    # FIXME(jathan): The writable serializer for `status` takes the
+    # status `name` (str) and not the `pk` (int). Do not validate this
+    # field right now, since we are asserting that it does create correctly.
+    #
+    # The test code for `utilities.testing.views.TestCase.model_to_dict()`
+    # needs to be enhanced to use the actual API serializers when `api=True`
+    validation_excluded_fields = ["status"]
+
     def setUp(self):
         super().setUp()
         self.statuses = Status.objects.get_for_model(Prefix)
         self.status_active = self.statuses.get(slug="active")
 
-    @classmethod
-    def setUpTestData(cls):
-
-        statuses = Status.objects.get_for_model(Prefix)
-
-        Prefix.objects.create(prefix=IPNetwork("192.168.1.0/24"), status=statuses[0])
-        Prefix.objects.create(prefix=IPNetwork("192.168.2.0/24"), status=statuses[0])
-        Prefix.objects.create(prefix=IPNetwork("192.168.3.0/24"), status=statuses[0])
-        Prefix.objects.create(prefix=IPNetwork("2001:db8:abcd::/80"), status=statuses[0])
-
-        # FIXME(jathan): The writable serializer for `status` takes the
-        # status `name` (str) and not the `pk` (int). Do not validate this
-        # field right now, since we are asserting that it does create correctly.
-        #
-        # The test code for `utilities.testing.views.TestCase.model_to_dict()`
-        # needs to be enhanced to use the actual API serializers when `api=True`
-        cls.validation_excluded_fields = ["status"]
-
     def test_list_available_prefixes(self):
         """
         Test retrieval of all available prefixes within a parent prefix.
         """
-        prefix = Prefix.objects.create(prefix=IPNetwork("192.0.2.0/24"))
-        Prefix.objects.create(prefix=IPNetwork("192.0.2.64/26"), status=self.status_active)
-        Prefix.objects.create(prefix=IPNetwork("192.0.2.192/27"), status=self.status_active)
+        prefix = Prefix.objects.ip_family(6).filter(prefix_length__lt=128).exclude(status__slug="container").first()
+        if prefix is None:
+            self.fail("Suitable prefix fixture not found")
         url = reverse("ipam-api:prefix-available-prefixes", kwargs={"pk": prefix.pk})
         self.add_permissions("ipam.view_prefix")
 
         # Retrieve all available IPs
         response = self.client.get(url, **self.header)
-        available_prefixes = ["192.0.2.0/26", "192.0.2.128/26", "192.0.2.224/27"]
+        available_prefixes = prefix.get_available_prefixes().iter_cidrs()
         for i, p in enumerate(response.data):
-            self.assertEqual(p["prefix"], available_prefixes[i])
+            self.assertEqual(p["prefix"], str(available_prefixes[i]))
 
     def test_create_single_available_prefix(self):
         """
         Test retrieval of the first available prefix within a parent prefix.
         """
-        vrf = VRF.objects.create(name="Test VRF 1", rd="1234")
-        prefix = Prefix.objects.create(
-            prefix=IPNetwork("192.0.2.0/28"),
-            vrf=vrf,
-            is_pool=True,
-            status=self.status_active,
-        )
+        # Find prefix with no child prefixes and large enough to create 4 child prefixes
+        for instance in Prefix.objects.filter(vrf__isnull=False):
+            if instance.get_child_prefixes().count() == 0 and instance.prefix.size > 2:
+                prefix = instance
+                break
+        else:
+            self.fail("Suitable prefix fixture not found")
         url = reverse("ipam-api:prefix-available-prefixes", kwargs={"pk": prefix.pk})
         self.add_permissions("ipam.add_prefix")
 
         # Create four available prefixes with individual requests
-        prefixes_to_be_created = [
-            "192.0.2.0/30",
-            "192.0.2.4/30",
-            "192.0.2.8/30",
-            "192.0.2.12/30",
-        ]
+        child_prefix_length = prefix.prefix_length + 2
+        prefixes_to_be_created = list(prefix.prefix.subnet(child_prefix_length))
         for i in range(4):
             data = {
-                "prefix_length": 30,
+                "prefix_length": child_prefix_length,
                 "status": "active",
                 "description": f"Test Prefix {i + 1}",
             }
             response = self.client.post(url, data, format="json", **self.header)
             self.assertHttpStatus(response, status.HTTP_201_CREATED)
-            self.assertEqual(response.data["prefix"], prefixes_to_be_created[i])
-            self.assertEqual(response.data["vrf"]["id"], str(vrf.pk))
+            self.assertEqual(response.data["prefix"], str(prefixes_to_be_created[i]))
+            self.assertEqual(response.data["vrf"]["id"], str(prefix.vrf.pk))
             self.assertEqual(response.data["description"], data["description"])
 
         # Try to create one more prefix
-        response = self.client.post(url, {"prefix_length": 30}, format="json", **self.header)
+        response = self.client.post(url, {"prefix_length": child_prefix_length}, format="json", **self.header)
         self.assertHttpStatus(response, status.HTTP_204_NO_CONTENT)
         self.assertIn("detail", response.data)
 
         # Try to create invalid prefix type
-        response = self.client.post(url, {"prefix_length": "30"}, format="json", **self.header)
+        response = self.client.post(url, {"prefix_length": str(child_prefix_length)}, format="json", **self.header)
         self.assertHttpStatus(response, status.HTTP_400_BAD_REQUEST)
         self.assertIn("prefix_length", response.data[0])
 
@@ -314,28 +264,36 @@ class PrefixTest(APIViewTestCases.APIViewTestCase):
         """
         Test the creation of available prefixes within a parent prefix.
         """
-        prefix = Prefix.objects.create(prefix=IPNetwork("192.0.2.0/28"), is_pool=True)
+        # Find prefix with no child prefixes and large enough to create 4 child prefixes
+        for instance in Prefix.objects.filter(vrf__isnull=False):
+            if instance.get_child_prefixes().count() == 0 and instance.prefix.size > 2:
+                prefix = instance
+                break
+        else:
+            self.fail("Suitable prefix fixture not found")
+
         url = reverse("ipam-api:prefix-available-prefixes", kwargs={"pk": prefix.pk})
         self.add_permissions("ipam.view_prefix", "ipam.add_prefix")
 
-        # Try to create five /30s (only four are available)
+        # Try to create five prefixes (only four are available)
+        child_prefix_length = prefix.prefix_length + 2
         data = [
-            {"prefix_length": 30, "description": "Test Prefix 1", "status": "active"},
-            {"prefix_length": 30, "description": "Test Prefix 2", "status": "active"},
-            {"prefix_length": 30, "description": "Test Prefix 3", "status": "active"},
-            {"prefix_length": 30, "description": "Test Prefix 4", "status": "active"},
-            {"prefix_length": 30, "description": "Test Prefix 5", "status": "active"},
+            {"prefix_length": child_prefix_length, "description": "Test Prefix 1", "status": "active"},
+            {"prefix_length": child_prefix_length, "description": "Test Prefix 2", "status": "active"},
+            {"prefix_length": child_prefix_length, "description": "Test Prefix 3", "status": "active"},
+            {"prefix_length": child_prefix_length, "description": "Test Prefix 4", "status": "active"},
+            {"prefix_length": child_prefix_length, "description": "Test Prefix 5", "status": "active"},
         ]
         response = self.client.post(url, data, format="json", **self.header)
         self.assertHttpStatus(response, status.HTTP_204_NO_CONTENT)
         self.assertIn("detail", response.data)
 
-        # Verify that no prefixes were created (the entire /28 is still available)
+        # Verify that no prefixes were created (the entire prefix is still available)
         response = self.client.get(url, **self.header)
         self.assertHttpStatus(response, status.HTTP_200_OK)
-        self.assertEqual(response.data[0]["prefix"], "192.0.2.0/28")
+        self.assertEqual(response.data[0]["prefix"], prefix.cidr_str)
 
-        # Create four /30s in a single request
+        # Create four prefixes in a single request
         response = self.client.post(url, data[:4], format="json", **self.header)
         self.assertHttpStatus(response, status.HTTP_201_CREATED)
         self.assertEqual(len(response.data), 4)
@@ -362,7 +320,7 @@ class PrefixTest(APIViewTestCases.APIViewTestCase):
         """
         Test retrieval of the first available IP address within a parent prefix.
         """
-        vrf = VRF.objects.create(name="Test VRF 1", rd="1234")
+        vrf = VRF.objects.first()
         prefix = Prefix.objects.create(
             prefix=IPNetwork("192.0.2.0/30"),
             vrf=vrf,
@@ -420,8 +378,9 @@ class ParallelPrefixTest(APITransactionTestCase):
         # 5 Prefixes
         requests = [{"prefix_length": 30, "description": f"Test Prefix {i}", "status": "active"} for i in range(1, 6)]
         url = reverse("ipam-api:prefix-available-prefixes", kwargs={"pk": prefix.pk})
-
+        logging.disable(logging.ERROR)
         self._do_parallel_requests(url, requests)
+        logging.disable(logging.NOTSET)
 
         prefixes = [str(o) for o in Prefix.objects.filter(prefix_length=30).all()]
         self.assertEqual(len(prefixes), len(set(prefixes)), "Duplicate prefixes should not exist")
@@ -432,9 +391,9 @@ class ParallelPrefixTest(APITransactionTestCase):
         # 8 IPs
         requests = [{"description": f"Test IP {i}", "status": "active"} for i in range(1, 9)]
         url = reverse("ipam-api:prefix-available-ips", kwargs={"pk": prefix.pk})
-
+        logging.disable(logging.ERROR)
         self._do_parallel_requests(url, requests)
-
+        logging.disable(logging.NOTSET)
         ips = [str(o) for o in IPAddress.objects.filter().all()]
         self.assertEqual(len(ips), len(set(ips)), "Duplicate IPs should not exist")
 
@@ -478,23 +437,13 @@ class IPAddressTest(APIViewTestCases.APIViewTestCase):
     }
     choices_fields = ["assigned_object_type", "role", "status"]
 
-    @classmethod
-    def setUpTestData(cls):
-
-        statuses = Status.objects.get_for_model(IPAddress)
-
-        IPAddress.objects.create(address=IPNetwork("192.168.0.1/24"), status=statuses[0])
-        IPAddress.objects.create(address=IPNetwork("192.168.0.2/24"), status=statuses[0])
-        IPAddress.objects.create(address=IPNetwork("192.168.0.3/24"), status=statuses[0])
-        IPAddress.objects.create(address=IPNetwork("2001:db8:abcd::20/128"), status=statuses[0])
-
-        # FIXME(jathan): The writable serializer for `status` takes the
-        # status `name` (str) and not the `pk` (int). Do not validate this
-        # field right now, since we are asserting that it does create correctly.
-        #
-        # The test code for `utilities.testing.views.TestCase.model_to_dict()`
-        # needs to be enhanced to use the actual API serializers when `api=True`
-        cls.validation_excluded_fields = ["status"]
+    # FIXME(jathan): The writable serializer for `status` takes the
+    # status `name` (str) and not the `pk` (int). Do not validate this
+    # field right now, since we are asserting that it does create correctly.
+    #
+    # The test code for `utilities.testing.views.TestCase.model_to_dict()`
+    # needs to be enhanced to use the actual API serializers when `api=True`
+    validation_excluded_fields = ["status"]
 
     def test_create_invalid_address(self):
         """Pass various invalid inputs and confirm they are rejected cleanly."""
@@ -512,8 +461,8 @@ class IPAddressTest(APIViewTestCases.APIViewTestCase):
         # Create the two outside NAT IP Addresses tied back to the single inside NAT address
         self.add_permissions("ipam.add_ipaddress")
         self.add_permissions("ipam.view_ipaddress")
-        nat_inside = IPAddress.objects.get(address="192.168.0.1/24")
-        # Create NAT outside with 192.168.0.1/24 IP as inside NAT
+        nat_inside = IPAddress.objects.filter(nat_outside_list__isnull=True).first()
+        # Create NAT outside with above address IP as inside NAT
         ip1 = self.client.post(
             self._get_list_url(),
             {"address": "192.0.2.1/24", "nat_inside": nat_inside.pk, "status": "active"},
@@ -572,16 +521,6 @@ class VLANGroupTest(APIViewTestCases.APIViewTestCase):
     }
     slug_source = "name"
 
-    @classmethod
-    def setUpTestData(cls):
-
-        vlan_groups = (
-            VLANGroup(name="VLAN Group 1", slug="vlan-group-1"),
-            VLANGroup(name="VLAN Group 2", slug="vlan-group-2"),
-            VLANGroup(name="VLAN Group 3", slug="vlan-group-3"),
-        )
-        VLANGroup.objects.bulk_create(vlan_groups)
-
 
 class VLANTest(APIViewTestCases.APIViewTestCase):
     model = VLAN
@@ -594,16 +533,7 @@ class VLANTest(APIViewTestCases.APIViewTestCase):
     @classmethod
     def setUpTestData(cls):
 
-        vlan_groups = (
-            VLANGroup.objects.create(name="VLAN Group 1", slug="vlan-group-1"),
-            VLANGroup.objects.create(name="VLAN Group 2", slug="vlan-group-2"),
-        )
-
-        statuses = Status.objects.get_for_model(VLAN)
-
-        VLAN.objects.create(name="VLAN 1", vid=1, group=vlan_groups[0], status=statuses[0])
-        VLAN.objects.create(name="VLAN 2", vid=2, group=vlan_groups[0], status=statuses[0])
-        VLAN.objects.create(name="VLAN 3", vid=3, group=vlan_groups[0], status=statuses[0])
+        vlan_groups = VLANGroup.objects.filter(site__isnull=False, location__isnull=False)[:2]
 
         # FIXME(jathan): The writable serializer for `status` takes the
         # status `name` (str) and not the `pk` (int). Do not validate this
@@ -617,20 +547,26 @@ class VLANTest(APIViewTestCases.APIViewTestCase):
             {
                 "vid": 4,
                 "name": "VLAN 4",
-                "group": vlan_groups[1].pk,
+                "group": vlan_groups[0].pk,
                 "status": "active",
+                "site": vlan_groups[0].site.pk,
+                "location": vlan_groups[0].location.pk,
             },
             {
                 "vid": 5,
                 "name": "VLAN 5",
-                "group": vlan_groups[1].pk,
+                "group": vlan_groups[0].pk,
                 "status": "active",
+                "site": vlan_groups[0].site.pk,
+                "location": vlan_groups[0].location.pk,
             },
             {
                 "vid": 6,
                 "name": "VLAN 6",
-                "group": vlan_groups[1].pk,
+                "group": vlan_groups[0].pk,
                 "status": "active",
+                "site": vlan_groups[0].site.pk,
+                "location": vlan_groups[0].location.pk,
             },
         ]
 
@@ -638,8 +574,7 @@ class VLANTest(APIViewTestCases.APIViewTestCase):
         """
         Attempt and fail to delete a VLAN with a Prefix assigned to it.
         """
-        vlan = VLAN.objects.first()
-        Prefix.objects.create(prefix=IPNetwork("192.0.2.0/24"), vlan=vlan)
+        vlan = VLAN.objects.filter(prefixes__isnull=False).first()
 
         self.add_permissions("ipam.delete_vlan")
         url = reverse("ipam-api:vlan-detail", kwargs={"pk": vlan.pk})
@@ -663,7 +598,7 @@ class ServiceTest(APIViewTestCases.APIViewTestCase):
 
     @classmethod
     def setUpTestData(cls):
-        site = Site.objects.create(name="Site 1", slug="site-1")
+        site = Site.objects.first()
         manufacturer = Manufacturer.objects.create(name="Manufacturer 1", slug="manufacturer-1")
         devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type 1")
         devicerole = DeviceRole.objects.create(name="Device Role 1", slug="device-role-1")
