@@ -1,10 +1,68 @@
-function jsify_form(context) {
-    this_context = $(context)
-    // Pagination
-    this_context.find('select#per_page').change(function() {
-        this.form.submit();
-    });
+/* ===========================
+*  Utility Functions
+*/
 
+// Slugify
+function slugify(s, num_chars) {
+    s = s.replace(/[^\-\.\w\s]/g, '');          // Remove unneeded chars
+    s = s.replace(/^[\s\.]+|[\s\.]+$/g, '');    // Trim leading/trailing spaces
+    s = s.replace(/[\-\.\s]+/g, '-');           // Convert spaces and decimals to hyphens
+    s = s.toLowerCase();                        // Convert to lowercase
+    // Declare `const slugify_prefer_underscores = true` globally if you want to use underscores instead of hyphens
+    if (typeof slugify_prefer_underscores !== "undefined") {
+        s = s.replace(/-/g, '_');
+    }
+    return s.substring(0, num_chars);           // Trim to first num_chars chars
+}
+
+// Parse URLs which may contain variable references to other field values
+function parseURL(url) {
+    var filter_regex = /\{\{([a-z_]+)\}\}/g;
+    var match;
+    var rendered_url = url;
+    var filter_field;
+    while (match = filter_regex.exec(url)) {
+        filter_field = $('#id_' + match[1]);
+        var custom_attr = $('option:selected', filter_field).attr('api-value');
+        if (custom_attr) {
+            rendered_url = rendered_url.replace(match[0], custom_attr);
+        } else if (filter_field.val()) {
+            rendered_url = rendered_url.replace(match[0], filter_field.val());
+        } else if (filter_field.attr('data-null-option')) {
+            rendered_url = rendered_url.replace(match[0], 'null');
+        }
+    }
+    return rendered_url
+}
+
+// Assign color picker selection classes
+function colorPickerClassCopy(data, container) {
+    if (data.element) {
+        // Swap the style
+        $(container).attr('style', $(data.element).attr("style"));
+    }
+    return data.text;
+}
+
+
+/* ===========================
+*  JS-ify Inputs
+*/
+
+// Static choice selection
+function initializeStaticChoiceSelection(context){
+    this_context = $(context);
+    this_context.find('.nautobot-select2-static').select2({
+        allowClear: true,
+        placeholder: "---------",
+        theme: "bootstrap",
+        width: "off"
+    });
+}
+
+// Static choice selection
+function initializeCheckboxes(context){
+    this_context = $(context);
     // "Toggle" checkbox for object lists (PK column)
     this_context.find('input:checkbox.toggle').click(function() {
         $(this).closest('table').find('input:checkbox[name=pk]:visible').prop('checked', $(this).prop('checked'));
@@ -23,29 +81,11 @@ function jsify_form(context) {
             $('input:checkbox.toggle, #select_all').prop('checked', false);
         }
     });
+}
 
-    // Enable hidden buttons when "select all" is checked
-    this_context.find('#select_all').click(function() {
-        if ($(this).is(':checked')) {
-            $('#select_all_box').find('button').prop('disabled', '');
-        } else {
-            $('#select_all_box').find('button').prop('disabled', 'disabled');
-        }
-    });
-
-    // Slugify
-    function slugify(s, num_chars) {
-        s = s.replace(/[^\-\.\w\s]/g, '');          // Remove unneeded chars
-        s = s.replace(/^[\s\.]+|[\s\.]+$/g, '');    // Trim leading/trailing spaces
-        s = s.replace(/[\-\.\s]+/g, '-');           // Convert spaces and decimals to hyphens
-        s = s.toLowerCase();                        // Convert to lowercase
-        // Declare `const slugify_prefer_underscores = true` globally if you want to use underscores instead of hyphens
-        if (typeof slugify_prefer_underscores !== "undefined") {
-            s = s.replace(/-/g, '_');
-        }
-        return s.substring(0, num_chars);           // Trim to first num_chars chars
-    }
-    var slug_field = $('#id_slug');
+function initializeSlugField(context){
+    this_context = $(context);
+    var slug_field = this_context.find('#id_slug');
     if (slug_field.length != 0) {
         var slug_source_arr = slug_field.attr('slug-source').split(" ");
         var slug_length = slug_field.attr('maxlength');
@@ -75,14 +115,12 @@ function jsify_form(context) {
                 }
             });
         }
-        $('button.reslugify').click(reslugify);
+        this_context.find('button.reslugify').click(reslugify);
     }
+}
 
-    // Bulk edit nullification
-    this_context.find('input:checkbox[name=_nullify]').click(function() {
-        $('#id_' + this.value).toggle('disabled');
-    });
-
+function initializeFormActionClick(context){
+    this_context = $(context);
     // Set formaction and submit using a link
     this_context.find('a.formaction').click(function(event) {
         event.preventDefault();
@@ -90,37 +128,19 @@ function jsify_form(context) {
         form.attr('action', $(this).attr('href'));
         form.submit();
     });
+}
 
-    // Parse URLs which may contain variable references to other field values
-    function parseURL(url) {
-        var filter_regex = /\{\{([a-z_]+)\}\}/g;
-        var match;
-        var rendered_url = url;
-        var filter_field;
-        while (match = filter_regex.exec(url)) {
-            filter_field = $('#id_' + match[1]);
-            var custom_attr = $('option:selected', filter_field).attr('api-value');
-            if (custom_attr) {
-                rendered_url = rendered_url.replace(match[0], custom_attr);
-            } else if (filter_field.val()) {
-                rendered_url = rendered_url.replace(match[0], filter_field.val());
-            } else if (filter_field.attr('data-null-option')) {
-                rendered_url = rendered_url.replace(match[0], 'null');
-            }
-        }
-        return rendered_url
-    }
+// Bulk edit nullification
+function initializeBulkEditNullification(context){
+    this_context = $(context);
+    this_context.find('input:checkbox[name=_nullify]').click(function() {
+        $('#id_' + this.value).toggle('disabled');
+    });
+}
 
-    // Assign color picker selection classes
-    function colorPickerClassCopy(data, container) {
-        if (data.element) {
-            // Swap the style
-            $(container).attr('style', $(data.element).attr("style"));
-        }
-        return data.text;
-    }
-
-    // Color Picker
+// Color Picker
+function initializeColorPicker(context){
+    this_context = $(context);
     this_context.find('.nautobot-select2-color-picker').select2({
         allowClear: true,
         placeholder: "---------",
@@ -129,19 +149,11 @@ function jsify_form(context) {
         templateSelection: colorPickerClassCopy,
         width: "off"
     });
+}
 
-    // Static choice selection
-    this_context.find('.nautobot-select2-static').select2({
-        allowClear: true,
-        placeholder: "---------",
-        theme: "bootstrap",
-        width: "off"
-    });
-
-    // API backed selection
-    // Includes live search and chained fields
-    // The `multiple` setting may be controlled via a data-* attribute
-
+// Dynamic Choice Selection
+function initializeDynamicChoiceSelection(context){
+    this_context = $(context);
     this_context.find('.nautobot-select2-api').select2({
         allowClear: true,
         placeholder: "---------",
@@ -171,11 +183,11 @@ function jsify_form(context) {
                     limit: 50,
                     offset: offset,
                 };
-                
+
                 // Set api_version
                 api_version = $(element).attr("data-api-version")
                 if(api_version)
-                    parameters["api_version"] = api_version
+                parameters["api_version"] = api_version
 
 
                 // Allow for controlling the brief setting from within APISelect
@@ -189,7 +201,17 @@ function jsify_form(context) {
                         $.each($.parseJSON(attr.value), function(index, value) {
                             // Referencing the value of another form field
                             if (value.startsWith('$')) {
-                                let ref_field = $('#id_' + value.slice(1));
+                                let element_id = $(element).attr("id")
+                                let ref_field;
+
+                                if(element_id.includes("id_form-")){
+                                    let id_prefix = element_id.match(/id_form-[0-9]+-/i, "")[0]
+                                    ref_field = $("#" + id_prefix + value.slice(1));
+                                }
+                                else {
+                                    ref_field = $('#id_' + value.slice(1));
+                                }
+
                                 if (ref_field.val() && ref_field.is(":visible")) {
                                     value = ref_field.val();
                                 } else if (ref_field.attr("required") && ref_field.attr("data-null-option")) {
@@ -211,6 +233,12 @@ function jsify_form(context) {
                     }
                 });
 
+                // Attach contenttype to parameters
+                contenttype = $(element).attr("data-contenttype");
+                if(contenttype){
+                    parameters["content_type"] = contenttype;
+                }
+
                 // This will handle params with multiple values (i.e. for list filter forms)
                 return $.param(parameters, true);
             },
@@ -226,10 +254,7 @@ function jsify_form(context) {
                         // Annotate hierarchical depth for MPTT objects
                         record.text = '--'.repeat(record._depth) + ' ' + record.text;
                     }
-                    if (record.tree_depth) {
-                        // Annotate hierarchical depth for django-tree-queries objects
-                        record.text = '--'.repeat(record.tree_depth) + ' ' + record.text;
-                    }
+
                     record.id = record[element.getAttribute('value-field')] || record.id;
                     if(element.getAttribute('disabled-indicator') && record[element.getAttribute('disabled-indicator')]) {
                         // The disabled-indicator equated to true, so we disable this option
@@ -280,8 +305,11 @@ function jsify_form(context) {
             }
         }
     });
+}
 
-    // Flatpickr selectors
+// Flatpickr selectors
+function initializeDateTimePicker(context){
+    this_context = $(context);
     this_context.find('.date-picker').flatpickr({
         allowInput: true
     });
@@ -298,11 +326,14 @@ function jsify_form(context) {
         noCalendar: true,
         time_24hr: true
     });
+}
 
-    // API backed tags
-    var tags = $('#id_tags.tagfield');
+function initializeTags(context){
+    this_context = $(context);
+    this_tag_field = this_context.find('#id_tags.tagfield')
+    var tags = this_tag_field;
     if (tags.length > 0 && tags.val().length > 0){
-        tags = $('#id_tags.tagfield').val().split(/,\s*/);
+        tags = this_tag_field.val().split(/,\s*/);
     } else {
         tags = [];
     }
@@ -314,8 +345,8 @@ function jsify_form(context) {
         }
     });
     // Replace the django issued text input with a select element
-    this_context.find('#id_tags.tagfield').replaceWith('<select name="tags" id="id_tags" class="form-control tagfield"></select>');
-    this_context.find('#id_tags.tagfield').select2({
+    this_tag_field.replaceWith('<select name="tags" id="id_tags" class="form-control tagfield"></select>');
+    this_tag_field.select2({
         tags: true,
         data: tag_objs,
         multiple: true,
@@ -343,7 +374,7 @@ function jsify_form(context) {
                 var results = $.map(data.results, function (obj) {
                     // If tag contains space add double quotes
                     if (/\s/.test(obj.name))
-                        obj.name = '"' + obj.name + '"'
+                    obj.name = '"' + obj.name + '"'
 
                     return {
                         id: obj.name,
@@ -362,8 +393,9 @@ function jsify_form(context) {
             }
         }
     });
-    this_context.find('#id_tags.tagfield').closest('form').submit(function(event){
+    this_tag_field.closest('form').submit(function(event){
         // django-taggit can only accept a single comma seperated string value
+        // TODO(bryan): the element find here should just be event.target
         var value = $('#id_tags.tagfield').val();
         if (value.length > 0){
             var final_tags = value.join(', ');
@@ -372,7 +404,10 @@ function jsify_form(context) {
             $('#id_tags.tagfield').append(option).trigger('change');
         }
     });
+}
 
+function initializeVLANModeSelection(context){
+    this_context = $(context);
     if( this_context.find('select#id_mode').length > 0 ) { // Not certain for the length check here as if none is find it should not apply the onChange
         this_context.find('select#id_mode').on('change', function () {
             if ($(this).val() == '') {
@@ -402,26 +437,105 @@ function jsify_form(context) {
         });
         this_context.find('select#id_mode').trigger('change');
     }
+}
 
-    // Scroll up an offset equal to the first nav element if a hash is present
-    // Cannot use '#navbar' because it is not always visible, like in small windows
-    function headerOffsetScroll() {
-        if (window.location.hash) {
-            // Short wait needed to allow the page to scroll to the element
-            setTimeout(function() {
-                window.scrollBy(0, -$('nav').height())
-            }, 10);
+function initializeMultiValueChar(context){
+    this_context = $(context);
+    this_context.find('.nautobot-select2-multi-value-char').select2({
+        allowClear: true,
+        tags: true,
+        theme: "bootstrap",
+        placeholder: "---------",
+        multiple: true,
+        width: "off",
+        "language": {
+            "noResults": function(){
+                return "Type something to add it as an option";
+            }
+        },
+    });
+}
+
+function initializeDynamicFilterForm(context){
+    this_context = $(context);
+    // Dynamic filter form
+    this_context.find(".lookup_type-select").bind("change", function(){
+        let parent_element = $(this).parents("tr")
+        let lookup_type = parent_element.find(".lookup_type-select")
+        let lookup_type_val = lookup_type.val()
+        let contenttype = lookup_type.attr("data-contenttype")
+        let lookup_value_element = parent_element.find(".lookup_value-input")
+
+        if(lookup_type_val){
+            $.ajax({
+                url: `/api/core/filterset-fields/lookup-value-dom-element/?field_name=${lookup_type_val}&content_type=${contenttype}`,
+                async: true,
+                contentType: 'application/json',
+                dataType: 'json',
+                type: 'GET',
+            }).done(function (response) {
+                newEl = $(response.dom_element)
+                newEl.addClass("lookup_value-input")
+                replaceEl(lookup_value_element, newEl)
+            }).fail(function (xhr, status, error) {
+                // Default to Input:text field if error occurs
+                createInput(lookup_value_element)
+            });
         }
-    }
 
-    // Account for the header height when hash-scrolling
-    window.addEventListener('load', headerOffsetScroll);
-    window.addEventListener('hashchange', headerOffsetScroll);
+    })
 
+    // On change of lookup_field or lookup_type field in filter form reset field value
+    this_context.find(".lookup_field-select, .lookup_type-select").on("change", function(){
+        let parent_element = $(this).parents("tr")
+        let lookup_field_element = parent_element.find(".lookup_field-select")
+        let lookup_type_element = parent_element.find(".lookup_type-select")
+        let lookup_value_element = parent_element.find(".lookup_value-input")
+
+        if ($(this)[0] == lookup_field_element[0]) {
+            lookup_type_element.val(null).trigger('change');
+        }
+        lookup_value_element.val(null).trigger('change')
+
+    })
+}
+
+function initializeSortableList(context){
+    this_context = $(context);
+    // Rearrange options within a <select> list
+    this_context.find('#move-option-up').bind('click', function() {
+        var select_id = '#' + $(this).attr('data-target');
+        $(select_id + ' option:selected').each(function () {
+            var newPos = $(select_id + ' option').index(this) - 1;
+            if (newPos > -1) {
+                $(select_id + ' option').eq(newPos).before("<option value='" + $(this).val() + "' selected='selected'>" + $(this).text() + "</option>");
+                $(this).remove();
+            }
+        });
+    });
+    this_context.find('#move-option-down').bind('click', function() {
+        var select_id = '#' + $(this).attr('data-target');
+        var countOptions = $(select_id + ' option').length;
+        var countSelectedOptions = $(select_id + ' option:selected').length;
+        $(select_id + ' option:selected').each(function () {
+            var newPos = $(select_id + ' option').index(this) + countSelectedOptions;
+            if (newPos < countOptions) {
+                $(select_id + ' option').eq(newPos).after("<option value='" + $(this).val() + "' selected='selected'>" + $(this).text() + "</option>");
+                $(this).remove();
+            }
+        });
+    });
+    this_context.find('#select-all-options').bind('click', function() {
+        var select_id = '#' + $(this).attr('data-target');
+        $(select_id + ' option').prop('selected',true);
+    });
+}
+
+function initializeImagePreview(context){
+    this_context = $(context);
     // Offset between the preview window and the window edges
     const IMAGE_PREVIEW_OFFSET_X = 20;
     const IMAGE_PREVIEW_OFFSET_Y = 10;
-
     // Preview an image attachment when the link is hovered over
     this_context.find('a.image-preview').on('mouseover', function(e) {
         // Twice the offset to account for all sides of the picture
@@ -456,37 +570,90 @@ function jsify_form(context) {
     this_context.find('a.image-preview').on('mouseout', function() {
         $('#image-preview-window').fadeOut('fast');
     });
-
-    // Rearrange options within a <select> list
-    this_context.find('#move-option-up').bind('click', function() {
-        var select_id = '#' + $(this).attr('data-target');
-        $(select_id + ' option:selected').each(function () {
-            var newPos = $(select_id + ' option').index(this) - 1;
-            if (newPos > -1) {
-                $(select_id + ' option').eq(newPos).before("<option value='" + $(this).val() + "' selected='selected'>" + $(this).text() + "</option>");
-                $(this).remove();
-            }
-        });
-    });
-    this_context.find('#move-option-down').bind('click', function() {
-        var select_id = '#' + $(this).attr('data-target');
-        var countOptions = $(select_id + ' option').length;
-        var countSelectedOptions = $(select_id + ' option:selected').length;
-        $(select_id + ' option:selected').each(function () {
-            var newPos = $(select_id + ' option').index(this) + countSelectedOptions;
-            if (newPos < countOptions) {
-                $(select_id + ' option').eq(newPos).after("<option value='" + $(this).val() + "' selected='selected'>" + $(this).text() + "</option>");
-                $(this).remove();
-            }
-        });
-    });
-    this_context.find('#select-all-options').bind('click', function() {
-        var select_id = '#' + $(this).attr('data-target');
-        $(select_id + ' option').prop('selected',true);
-    });
-
 }
 
+function initializeSelectAllForm(context){
+    this_context = $(context);
+    this_context.find('#select_all').click(function() {
+        if ($(this).is(':checked')) {
+            $('#select_all_box').find('button').prop('disabled', '');
+        } else {
+            $('#select_all_box').find('button').prop('disabled', 'disabled');
+        }
+    });
+}
+
+function initializeResultPerPageSelection(context){
+    this_context = $(context);
+    this_context.find('select#per_page').change(function() {
+        this.form.submit();
+    });
+}
+
+function replaceEl(replaced_el, replacing_el) {
+    parent = replaced_el.parent()
+    parent.html(replacing_el)
+    initializeInputs(parent)
+}
+
+function initializeInputs(context) {
+    this_context = $(context);
+    initializeStaticChoiceSelection(this_context)
+    initializeCheckboxes(this_context)
+    initializeSlugField(this_context)
+    initializeFormActionClick(this_context)
+    initializeBulkEditNullification(this_context)
+    initializeColorPicker(this_context)
+    initializeDynamicChoiceSelection(this_context)
+    initializeDateTimePicker(this_context)
+    initializeTags(this_context)
+    initializeVLANModeSelection(this_context)
+    initializeSortableList(this_context)
+    initializeImagePreview(this_context)
+    initializeDynamicFilterForm(this_context)
+    initializeSelectAllForm(this_context)
+    initializeMultiValueChar(this_context)
+}
+
+function jsify_form(context) {
+    this_context = $(context);
+    // Pagination
+    initializeInputs(this_context)
+}
+
+/* =======
+*  Input Creators
+*/
+
+
+function createInput(element){
+    input_field = `
+    <input
+    type="text"
+    name="${element.attr('name')}"
+    class="lookup_value-input form-control"
+    id="${element.attr('id')}"
+    />`
+    replaceEl(element, input_field)
+}
+
+
 $(document).ready((e) => {
     jsify_form(this.document);
-})
\ No newline at end of file
+    initializeResultPerPageSelection(this.document);
+})
+
+// Scroll up an offset equal to the first nav element if a hash is present
+// Cannot use '#navbar' because it is not always visible, like in small windows
+function headerOffsetScroll() {
+    if (window.location.hash) {
+        // Short wait needed to allow the page to scroll to the element
+        setTimeout(function() {
+            window.scrollBy(0, -$('nav').height())
+        }, 10);
+    }
+}
+
+// Account for the header height when hash-scrolling
+window.addEventListener('load', headerOffsetScroll);
+window.addEventListener('hashchange', headerOffsetScroll);
\ No newline at end of file
