@@ -16,15 +16,17 @@ from django_filters.utils import get_model_field, resolve_field
 from mptt.models import MPTTModel
 from tree_queries.models import TreeNode
 
+from nautobot.dcim.fields import MACAddressCharField
 from nautobot.dcim.forms import MACAddressField
 from nautobot.extras.models import Tag
 from nautobot.utilities.constants import (
     FILTER_CHAR_BASED_LOOKUP_MAP,
     FILTER_NEGATION_LOOKUP_MAP,
     FILTER_NUMERIC_BASED_LOOKUP_MAP,
-    FILTER_TREENODE_NEGATION_LOOKUP_MAP,
 )
-from nautobot.utilities.forms.fields import MultiMatchModelMultipleChoiceField
+from nautobot.utilities.forms.fields import MultiMatchModelMultipleChoiceField, MultiValueCharField
+from nautobot.utilities.utils import flatten_iterable
+
 
 from taggit.managers import TaggableManager
 
@@ -51,12 +53,12 @@ def multivalue_field_factory(field_class):
 
             return [
                 # Only append non-empty values (this avoids e.g. trying to cast '' as an integer)
-                super(field_class, self).to_python(v)
+                super(field_class, self).to_python(v)  # pylint: disable=bad-super-call
                 for v in value
                 if v
             ]
 
-    return type("MultiValue{}".format(field_class.__name__), (NewField,), dict())
+    return type(f"MultiValue{field_class.__name__}", (NewField,), {})
 
 
 #
@@ -70,7 +72,7 @@ def multivalue_field_factory(field_class):
 
 
 class MultiValueCharFilter(django_filters.CharFilter, django_filters.MultipleChoiceFilter):
-    field_class = multivalue_field_factory(forms.CharField)
+    field_class = MultiValueCharField
 
 
 class MultiValueDateFilter(django_filters.DateFilter, django_filters.MultipleChoiceFilter):
@@ -157,7 +159,7 @@ class NullableCharFieldFilter(django_filters.CharFilter):
     def filter(self, qs, value):
         if value != settings.FILTERS_NULL_CHOICE_VALUE:
             return super().filter(qs, value)
-        qs = self.get_method(qs)(**{"{}__isnull".format(self.field_name): True})
+        qs = self.get_method(qs)(**{f"{self.field_name}__isnull": True})
         return qs.distinct() if self.distinct else qs
 
 
@@ -350,9 +352,9 @@ class MappedPredicatesFilterMixin:
         super().__init__(*args, **kwargs)
 
         # Generate the query with a sentinel value to validate it and surface parse errors.
-        self.generate_query(self.filter_predicates, value="")
+        self.generate_query(value="", filter_predicates=self.filter_predicates)
 
-    def generate_query(self, filter_predicates, value):
+    def generate_query(self, value, filter_predicates=None, **kwargs):
         """
         Given a mapping of `filter_predicates` and a `value`, return a `Q` object for 2-tuple of
         predicate=value.
@@ -400,7 +402,7 @@ class MappedPredicatesFilterMixin:
             return qs
 
         # Evaluate the query and stash it for later use (such as introspection or debugging)
-        query = self.generate_query(self.filter_predicates, value)
+        query = self.generate_query(value=value, filter_predicates=self.filter_predicates)
         qs = self.get_method(qs)(query)
         self._most_recent_query = query
         return qs.distinct()
@@ -444,18 +446,12 @@ class NaturalKeyOrPKMultipleChoiceFilter(django_filters.ModelMultipleChoiceFilte
             v = str(v)  # Cast possible UUID instance to a string
             is_pk = True
 
-        # If it's not a pk and not a list/qs generate then it's a slug and the filter predicate
-        # needs to be nested (e.g. `{"site__slug": "ams01"}`) so that it can be useable in `Q`
-        # objects.
-        #
-        # FIXME(jathan): It feels weird to have a list/qs make its way to `get_filter_predicate()`
-        # which if you inspect the source for `django_filters.MultipleChoiceFilter.filter()` should
-        # be handling those to generate singular filter predicates and concatentating them.
-        if not is_pk and not isinstance(v, (list, models.QuerySet)):
+        # If it's not a pk, then it's a slug and the filter predicate needs to be nested (e.g.
+        # `{"site__slug": "ams01"}`) so that it can be usable in `Q` objects.
+        if not is_pk:
             name = f"{self.field_name}__{self.field.to_field_name}"
-        # Otherwise just trust the field_name
         else:
-            logger.debug("UUID or list/qs detected: Filtering using field name")
+            logger.debug("UUID detected: Filtering using field name")
             name = self.field_name
 
         if name and self.lookup_expr != django_filters.conf.settings.DEFAULT_LOOKUP_EXPR:
@@ -492,10 +488,13 @@ class TreeNodeMultipleChoiceFilter(NaturalKeyOrPKMultipleChoiceFilter):
     """
 
     def __init__(self, *args, **kwargs):
-        kwargs.setdefault("lookup_expr", "in")
+        kwargs.pop("lookup_expr", None)  # Disallow overloading of `lookup_expr`.
         super().__init__(*args, **kwargs)
 
-    def filter(self, qs, value):
+    def generate_query(self, value, qs=None, **kwargs):
+        """
+        Given a filter value, return a `Q` object that accounts for nested tree node descendants.
+        """
         if value:
             if any(isinstance(node, TreeNode) for node in value):
                 # django-tree-queries
@@ -505,7 +504,34 @@ class TreeNodeMultipleChoiceFilter(NaturalKeyOrPKMultipleChoiceFilter):
                 value = [
                     node.get_descendants(include_self=True) if not isinstance(node, str) else node for node in value
                 ]
-        return super().filter(qs, value)
+
+        # This new_value is going to be a list of querysets that needs to be flattened.
+        value = list(flatten_iterable(value))
+
+        # Construct a list of filter predicates that will be used to generate the Q object.
+        predicates = []
+        for obj in value:
+            # Try to get the `to_field_name` (e.g. `slug`) or just pass the object through.
+            val = getattr(obj, self.field.to_field_name, obj)
+            if val == self.null_value:
+                val = None
+            predicates.append(self.get_filter_predicate(val))
+
+        # Construct a nested OR query from the list of filter predicates derived from the flattened
+        # listed of descendant objects.
+        query = models.Q()
+        for predicate in predicates:
+            query |= models.Q(**predicate)
+
+        return query
+
+    def filter(self, qs, value):
+        if value in EMPTY_VALUES:
+            return qs
+
+        # Fetch the generated Q object and filter the incoming qs with it before passing it along.
+        query = self.generate_query(value)
+        return self.get_method(qs)(query)
 
 
 #
@@ -540,7 +566,7 @@ class BaseFilterSet(django_filters.FilterSet):
             models.TimeField: {"filter_class": MultiValueTimeFilter},
             models.URLField: {"filter_class": MultiValueCharFilter},
             models.UUIDField: {"filter_class": MultiValueUUIDFilter},
-            MACAddressField: {"filter_class": MultiValueMACAddressFilter},
+            MACAddressCharField: {"filter_class": MultiValueMACAddressFilter},
             TaggableManager: {"filter_class": TagFilter},
         }
     )
@@ -559,10 +585,6 @@ class BaseFilterSet(django_filters.FilterSet):
         ):
             lookup_map = FILTER_NUMERIC_BASED_LOOKUP_MAP
 
-        elif isinstance(existing_filter, (TreeNodeMultipleChoiceFilter,)):
-            # TreeNodeMultipleChoiceFilter only support negation but must maintain the `in` lookup expression
-            lookup_map = FILTER_TREENODE_NEGATION_LOOKUP_MAP
-
         # These filter types support only negation
         elif isinstance(
             existing_filter,
@@ -570,12 +592,15 @@ class BaseFilterSet(django_filters.FilterSet):
                 django_filters.ModelChoiceFilter,
                 django_filters.ModelMultipleChoiceFilter,
                 TagFilter,
+                TreeNodeMultipleChoiceFilter,
             ),
         ):
             lookup_map = FILTER_NEGATION_LOOKUP_MAP
+
         # These filter types support only negation
         elif existing_filter.extra.get("choices"):
             lookup_map = FILTER_NEGATION_LOOKUP_MAP
+
         elif isinstance(
             existing_filter,
             (
@@ -618,7 +643,7 @@ class BaseFilterSet(django_filters.FilterSet):
 
         # Create new filters for each lookup expression in the map
         for lookup_name, lookup_expr in lookup_map.items():
-            new_filter_name = "{}__{}".format(filter_name, lookup_name)
+            new_filter_name = f"{filter_name}__{lookup_name}"
 
             try:
                 if filter_name in cls.declared_filters:
@@ -663,7 +688,7 @@ class BaseFilterSet(django_filters.FilterSet):
 
         if new_filter_name in cls.base_filters:
             raise AttributeError(
-                "There was a conflict with filter `%s`, the custom filter was ignored." % new_filter_name
+                f"There was a conflict with filter `{new_filter_name}`, the custom filter was ignored."
             )
 
         cls.base_filters[new_filter_name] = new_filter_field
